@@ -19,7 +19,14 @@ score_to_track: str
     and we are allowed to change tau coefficient further; otherwise we revert back
     to the last "safe" value and stop
 
-    More formal definition of "sort of decreasing": if we divide a curve into two parts like so:
+    'sort of decreasing' performs best with `PerplexityScore`,
+    and all scores which behave like perplexity
+    (nonnegative, and which should decrease when a model gets better).
+    If you want to track a different kind of score,
+    it is recommended to use `score_controller` parameter
+
+    More formal definition of "sort of decreasing":
+    if we divide a curve into two parts like so:
 
 
         ##################################### 
@@ -46,12 +53,18 @@ score_to_track: str
                          |     right part   | 
 
     then the right part is no higher than 5% of global minimum
-    (you can change 5% if you like by adjusting `fraction_threshold`
-     in `is_score_out_of_control` function)
+    (you can change 5% if you like by adjusting `fraction_threshold` parameter)
 
-    If score_to_track is None, then `ControllerAgent` will never stop
+    If `score_to_track` is None and `score_controller` is None,
+    then `ControllerAgent` will never stop
     (useful for e.g. decaying coefficients)
-
+fraction_threshold: float
+    Threshold to control a score by 'sort of decreasing' metric
+score_controller: BaseScoreController
+    Custom score controller
+    In case of 'sort of decreasing' is not proper to control score,
+    you are able to create custom Score Controller 
+    inherited from `BaseScoreController`.
 tau_converter: str or callable
     Notably, def-style functions and lambda functions are allowed
     If it is function, then it should accept four arguments:
@@ -109,58 +122,112 @@ max_iter: numeric
     that way agent will continue operating even outside this `RegularizationControllerCube`
 """  # noqa: W291
 
-from .base_cube import BaseCube
-from ..rel_toolbox_lite import count_vocab_size, handle_regularizer
+import warnings
+from copy import deepcopy
+from dataclasses import dataclass
+from numbers import Number
+from typing import (
+    Callable,
+    List,
+    Optional,
+    Union,
+)
 
 import numexpr as ne
-import warnings
-from dill.source import getsource
-from copy import deepcopy
 import numpy as np
+from dill.source import getsource
 
+from .base_cube import BaseCube
+from ..models.base_regularizer import BaseRegularizer
+from ..rel_toolbox_lite import count_vocab_size, handle_regularizer
 
 W_HALT_CONTROL = "Process of dynamically changing tau was stopped at {} iteration"
 W_MAX_ITERS = "Maximum number of iterations is exceeded; turning off"
 
 
-def is_score_out_of_control(model, score_name, fraction_threshold=0.05):
+@dataclass
+class OutOfControlAnswer:
+    answer: bool
+    error_message: Optional[str] = None
+
+
+class BaseScoreController:
+    def __init__(self, score_name):
+        self.score_name = score_name
+
+    def get_score_values(self, model):
+        if self.score_name not in model.scores:  # case of None is handled here as well
+            return None
+
+        vals = model.scores[self.score_name]
+
+        if len(vals) == 0:
+            return None
+
+        return vals
+
+    def __call__(self, model):
+        values = self.get_score_values(model)
+
+        if values is None:
+            return False
+
+        try:
+            out_of_control_result = self.is_out_of_control(values)
+        except Exception as ex:
+            raise ValueError(
+                f"An error occurred while controlling {self.score_name}!"
+                f" Message: {ex}. Score values: {values}"
+            )
+
+        if out_of_control_result.error_message is not None:
+            warnings.warn(out_of_control_result.error_message)
+
+        return out_of_control_result.answer
+
+    def is_out_of_control(self, values: List[float]) -> OutOfControlAnswer:
+        raise NotImplementedError
+
+
+class PerplexityScoreController(BaseScoreController):
     """
-    Returns True if score isn't 'sort of decreasing' anymore.
-
-    See docstring for RegularizationControllerCube for details
-
-    Parameters
-    ----------
-    model : TopicModel
-    score_name : str or None
-    fraction_threshold : float
-
-    Returns
-    -------
-    bool
-
+    Controller is proper to control the Perplexity score.
+    For others, please ensure for yourself.
     """
+    DEFAULT_FRACTION_THRESHOLD = 0.05
 
-    if score_name not in model.scores:  # case of None is handled here as well
-        return False
+    def __init__(self, score_name, fraction_threshold=DEFAULT_FRACTION_THRESHOLD):
+        super().__init__(score_name)
+        self.fraction_threshold = fraction_threshold
 
-    vals = model.scores[score_name]
-    if len(vals) == 0:
-        return False
+    def is_out_of_control(self, values: List[float]):
+        idxmin = np.argmin(values)
 
-    idxmin = np.argmin(vals)
+        if idxmin == len(values):  # score is monotonically decreasing
+            return False
 
-    if idxmin == len(vals):  # score is monotonically decreasing
-        return False
-    maxval = max(vals[idxmin:])
-    minval = vals[idxmin]
-    answer = ((maxval - minval)/abs(minval) - 1.0) > fraction_threshold
-    if answer:
-        msg = (f"Score {score_name} is too high: during training the value {maxval}"
-               f" passed a treshold of {(1 + fraction_threshold) * minval}"
-               f" (estimate is based on {idxmin} iteration)")
-        warnings.warn(msg)
-    return answer
+        right_maxval = max(values[idxmin:])
+        minval = values[idxmin]
+
+        if minval <= 0:
+            raise ValueError(
+                f'Score "{self.score_name}" has min_value = {minval} which is <= 0.'
+                f' This control scheme is using to control scores acting like Perplexity.'
+                f' Ensure you control the Perplexity score or write your own controller!'
+            )
+
+        answer = (right_maxval - minval) / minval > self.fraction_threshold
+
+        if answer:
+            return OutOfControlAnswer(
+                answer=answer,
+                error_message=(
+                    f"Score {self.score_name} is too high!"
+                    f" Right max value: {right_maxval}, min value: {minval}"
+                ),
+            )
+
+        return OutOfControlAnswer(answer=answer)
 
 
 class ControllerAgent:
@@ -172,8 +239,10 @@ class ControllerAgent:
     Each agent is described by:
 
     * reg_name: the name of regularizer having `tau` which needs to be changed
-    * score_to_track: score providing control of the callback execution
     * tau_converter: function or string describing how to get new `tau` from old `tau`
+    * score_to_track: score name providing control of the callback execution
+    * fraction_threshold: threshold to control score_to_track
+    * score_controller: custom score controller providing control of the callback execution
     * local_dict: dictionary containing values of several variables,
             most notably, `user_value`
     * is_working:
@@ -183,37 +252,125 @@ class ControllerAgent:
 
     See top-level docstring for details.
     """
-    def __init__(self, reg_name, score_to_track, tau_converter, max_iters, local_dict=None):
+
+    def __init__(
+            self,
+            reg_name: str,
+            tau_converter: Callable or str,
+            max_iters: int or float,
+            score_to_track: Union[str, List[str], None] = None,
+            fraction_threshold: Union[float, List[float], None] = None,
+            score_controller: Union[BaseScoreController, List[BaseScoreController], None] = None,
+            local_dict: dict = None):
         """
 
         Parameters
         ----------
-        reg_name : str
-        score_to_track : str, list of str or None
-        tau_converter : callable or str
-        local_dict : dict
-        max_iters : int or float
-            Agent will stop changing tau after `max_iters` iterations
+        reg_name
+        tau_converter
+        max_iters
+            Agent will stop changing tau after `max_iters` iterations,
             `max_iters` could be `float("NaN")` and `float("inf")` values:
             that way agent will continue operating even outside this `RegularizationControllerCube`
+        score_to_track
+            Name of score to track.
+            Please, use this definition to track only scores of type PerplexityScore.
+            In other cases we recommend you to write you own ScoreController
+        fraction_threshold
+            Uses to define threshold to control PerplexityScore
+            Default value is 0.05.
+            If `fraction_threshold` is a list, it should be of the same length, as `score_to_track`.
+        score_controller
+            Score controller or controllers.
+            One can use this parameter for scores other than Perplexity
+            (or other scores that behave like Perplexity).
+            This is a more flexible and customizable way to control scores.
+        local_dict
         """
         if local_dict is None:
             local_dict = dict()
 
         self.reg_name = reg_name
         self.tau_converter = tau_converter
-        if isinstance(score_to_track, list):
-            self.score_to_track = score_to_track
-        elif isinstance(score_to_track, str):
-            self.score_to_track = [score_to_track]
-        else:
-            self.score_to_track = []
+
+        scores_to_track = self._validate_score_to_track(score_to_track)
+        fraction_thresholds = self._validate_fraction_threshold(
+            fraction_threshold, required_length=len(scores_to_track)
+        )
+
+        assert len(scores_to_track) == len(fraction_thresholds)
+
+        perplexity_like_score_controllers = [
+            PerplexityScoreController(name, threshold)
+            for (name, threshold) in zip(scores_to_track, fraction_thresholds)
+        ]
+
+        self.score_controllers = list()
+        self.score_controllers.extend(perplexity_like_score_controllers)
+        self.score_controllers.extend(
+            self._validate_score_controller(score_controller)
+        )
 
         self.is_working = True
         self.local_dict = local_dict
         self.tau_history = []
         self.max_iters = max_iters
 
+    @staticmethod
+    def _validate_score_to_track(
+            score_to_track: Union[str, List[str], None]) -> List[str]:
+
+        if isinstance(score_to_track, list):
+            return score_to_track
+        if score_to_track is None:
+            return list()
+        if isinstance(score_to_track, str):
+            return [score_to_track]
+
+        raise TypeError(f'Wrong type of `score_to_track`: "{type(score_to_track)}"!')
+
+    @staticmethod
+    def _validate_fraction_threshold(
+            fraction_threshold: Union[float, List[float], None],
+            required_length: int,
+    ) -> List[float]:
+
+        if fraction_threshold is None:
+            return [PerplexityScoreController.DEFAULT_FRACTION_THRESHOLD] * required_length
+        if isinstance(fraction_threshold, Number):
+            return [float(fraction_threshold)] * required_length
+
+        if not isinstance(fraction_threshold, list):
+            raise TypeError(
+                f'Wrong type of `fraction_threshold`: "{type(fraction_threshold)}"!'
+            )
+
+        if len(fraction_threshold) != required_length:
+            raise ValueError(
+                f'Wrong length of `fraction_threshold`: {len(fraction_threshold)}!'
+                f' Expected the length to be equal to {required_length}.'
+            )
+
+        return fraction_threshold
+
+    @staticmethod
+    def _validate_score_controller(
+            score_controller: Union[BaseScoreController, List[BaseScoreController], None]
+    ) -> List[BaseScoreController]:
+
+        if score_controller is None:
+            return list()
+
+        elif isinstance(score_controller, BaseScoreController):
+            return [score_controller]
+
+        elif (not isinstance(score_controller, list) or not all(
+                isinstance(score, BaseScoreController) for score in score_controller)):
+            raise TypeError(f'Wrong type of `score_controller`: "{type(score_controller)}"!')
+
+        else:
+            return score_controller
+
     def _convert_tau(self):
         """ """
         if isinstance(self.tau_converter, str):
@@ -244,7 +401,7 @@ class ControllerAgent:
             Note that zero means "cube just started", not "the model is brand new"
 
         """
-        current_tau = model.regularizers[self.reg_name].tau
+        current_tau = model.get_regularizer(self.reg_name).tau
         self.tau_history.append(current_tau)
         self.local_dict["prev_tau"] = current_tau
         self.local_dict["cur_iter"] = cur_iter
@@ -258,14 +415,14 @@ class ControllerAgent:
 
         if self.is_working:
             should_stop = any(
-                is_score_out_of_control(model, score) for score in self.score_to_track
+                score_controller(model) for score_controller in self.score_controllers
             )
             if should_stop:
                 warnings.warn(W_HALT_CONTROL.format(len(self.tau_history)))
                 self.is_working = False
-                model.regularizers[self.reg_name].tau = self._find_safe_tau()
+                model.get_regularizer(self.reg_name).tau = self._find_safe_tau()
             else:
-                model.regularizers[self.reg_name].tau = self._convert_tau()
+                model.get_regularizer(self.reg_name).tau = self._convert_tau()
 
 
 class RegularizationControllerCube(BaseCube):
@@ -283,26 +440,33 @@ class RegularizationControllerCube(BaseCube):
             regularizers params
             each dict should contain the following fields: 
                 ("reg_name" or "regularizer"),
-                "score_to_track" (optional),
                 "tau_converter",
+                "score_to_track" (optional),
+                "fraction_threshold" (optional),
+                "score_controller" (optional),
                 "user_value_grid"
                 See top-level docstring for details.
             Examples:
 
                     >>  {"regularizer": artm.regularizers.<...>,
-                    >>   "score_to_track": "PerplexityScore@all",
                     >>   "tau_converter": "prev_tau * user_value",
+                    >>   "score_to_track": "PerplexityScore@all",
+                    >>   "fraction_threshold": 0.1,
                     >>   "user_value_grid": [0.5, 1, 2]}
 
 
             -----------
 
                     >>  {"reg_name": "decorrelator_for_ngramms",
-                    >>   "score_to_track": None,
                     >>   "tau_converter": (
                     >>       lambda initial_tau, prev_tau, cur_iter, user_value:
                     >>       initial_tau * (cur_iter % 2) + user_value
                     >>   )
+                    >>   "score_to_track": None,
+                    >>   "fraction_threshold": None,
+                    >>   "score_controller": [
+                    >>       PerplexityScoreController("PerplexityScore@all", 0.1)
+                    >>   ],
                     >>   "user_value_grid": [0, 1]}
 
         reg_search : str
@@ -329,9 +493,9 @@ class RegularizationControllerCube(BaseCube):
                          separate_thread=separate_thread)
         self._relative = use_relative_coefficients
         self.data_stats = None
-        self.raw_parameters = parameters
         if isinstance(parameters, dict):
             parameters = [parameters]
+        self.raw_parameters = parameters
         self._convert_parameters(parameters)
 
     def _convert_parameters(self, all_parameters):
@@ -392,22 +556,29 @@ class RegularizationControllerCube(BaseCube):
 
         for (agent_blueprint_template, field_name, current_user_value) in one_model_parameter:
             agent_blueprint = dict(agent_blueprint_template)
-            if agent_blueprint["reg_name"] is None:
-                regularizer = agent_blueprint["regularizer"]
-                new_regularizer = deepcopy(regularizer)
-                handle_regularizer(
-                    self._relative,
-                    new_model,
-                    new_regularizer,
-                    self.data_stats,
-                )
-                agent_blueprint["reg_name"] = new_regularizer.name
-            else:
-                if agent_blueprint['reg_name'] not in new_model.regularizers.data:
+            if agent_blueprint.get("reg_name") is not None:
+                reg_name = agent_blueprint['reg_name']
+
+                if reg_name not in new_model.all_regularizers:
                     error_msg = (f"Regularizer {agent_blueprint['reg_name']} does not exist. "
                                  f"Cannot be modified.")
                     raise ValueError(error_msg)
 
+            elif agent_blueprint.get("regularizer") is not None:
+                regularizer = agent_blueprint["regularizer"]
+                new_regularizer = deepcopy(regularizer)
+                if isinstance(regularizer, BaseRegularizer):
+                    new_model.custom_regularizers[new_regularizer.name] = new_regularizer
+                else:  # classic bigARTM regularizer, attempt to relativize it's coefficients
+                    handle_regularizer(
+                        self._relative,
+                        new_model,
+                        new_regularizer,
+                        self.data_stats,
+                    )
+                agent_blueprint["reg_name"] = new_regularizer.name
+            else:
+                raise ValueError("Either 'reg_name' or 'regularizer' should be set")
             agent_blueprint['local_dict']['user_value'] = current_user_value
             # ControllerAgent needs only reg_name in constructor
             agent_blueprint.pop("regularizer")
