@@ -8,6 +8,7 @@ import warnings
 from collections import defaultdict
 from enum import Enum, IntEnum, auto
 from typing import (
+    Callable,
     Dict,
     List,
     Optional,
@@ -36,16 +37,16 @@ class ComputationMethod(IntEnum):
 
     Attributes
     ----------
-        SEGMENT_LENGTH :
-            Estimate the length of topic segments
-        SEGMENT_WEIGHT :
-            Estimate the weight of topic segment
-            (weight - sum of specificities for the topic over words in segment)
-        SUM_OVER_WINDOW :
-            Sum of specificities for the topic over words in given window.
-            The process is as follows:
-            word of the topic is found in text, it is the center of the first window;
-            next word of the topic is found (outside of the previous window), window; etc
+    SEGMENT_LENGTH :
+        Estimate the length of topic segments
+    SEGMENT_WEIGHT :
+        Estimate the weight of topic segment
+        (weight - sum of specificities for the topic over words in segment)
+    SUM_OVER_WINDOW :
+        Sum of specificities for the topic over words in given window.
+        The process is as follows:
+        word of the topic is found in text, it is the center of the first window;
+        next word of the topic is found (outside of the previous window), window; etc
     """
     SEGMENT_LENGTH = auto()
     SEGMENT_WEIGHT = auto()
@@ -58,10 +59,10 @@ class WordTopicRelatednessType(IntEnum):
 
     Attributes
     ----------
-        PWT :
-            p(w | t)
-        PTW :
-            p(t | w)
+    PWT :
+        p(w | t)
+    PTW :
+        p(t | w)
     """
     PWT = auto()
     PTW = auto()
@@ -77,14 +78,14 @@ class SpecificityEstimationMethod(IntEnum):
 
     Attributes
     ----------
-        NONE :
-            Don't try to estimate specificity_estimation, return the probability as is
-        MAXIMUM :
-            From probability, corresponding to word and topic,
-            extract *maximum* among probabilities for the word and other topics
-        AVERAGE :
-            From probability, corresponding to word and topic,
-            extract *average* among probabilities for the word and other topics
+    NONE :
+        Don't try to estimate specificity_estimation, return the probability as is
+    MAXIMUM :
+        From probability, corresponding to word and topic,
+        extract *maximum* among probabilities for the word and other topics
+    AVERAGE :
+        From probability, corresponding to word and topic,
+        extract *average* among probabilities for the word and other topics
     """
     NONE = auto()
     MAXIMUM = auto()
@@ -105,6 +106,7 @@ class IntratextCoherenceScore(BaseScore):
             self,
             dataset: Union[Dataset, str],
             name: str = None,
+            should_compute: Callable[[int], bool] = None,
             keep_dataset_in_memory: bool = None,
             keep_dataset: bool = True,
             documents: List[str] = None,
@@ -189,7 +191,7 @@ class IntratextCoherenceScore(BaseScore):
         >>> topic_model._fit(dataset.get_batch_vectorizer(), num_iterations=num_iterations)
         """
         # TODO: word_topic_relatedness seems to be connected with TopTokensViewer stuff
-        super().__init__(name=name)
+        super().__init__(name=name, should_compute=should_compute)
 
         self._keep_dataset = keep_dataset
 
@@ -363,7 +365,7 @@ class IntratextCoherenceScore(BaseScore):
 
         return score
 
-    def call(self, model: BaseModel) -> float:
+    def call(self, model: BaseModel, **kwargs) -> float:
         if (self._current_iteration - self._start_fit_iteration) % self._fit_iteration_step != 0:
             self._current_iteration += 1
 
@@ -468,8 +470,8 @@ class IntratextCoherenceScore(BaseScore):
         elif self._specificity_estimation_method == SpecificityEstimationMethod.AVERAGE:
             word_topic_probs[:] = (
                 word_topic_probs.values -
-                    np.sum(word_topic_probs.values, axis=1, keepdims=True) /  # noqa: line alignment
-                        max(word_topic_probs.shape[1], 1)
+                    np.sum(word_topic_probs.values, axis=1, keepdims=True) /  # noqa E131
+                        max(word_topic_probs.shape[1], 1)  # noqa E131
             )
 
         elif self._specificity_estimation_method == SpecificityEstimationMethod.MAXIMUM:
