@@ -6,7 +6,7 @@ from scipy.spatial.distance import cdist as sp_cdist
 from typing import Callable
 
 from .base_viewer import BaseViewer
-from ..cooking_machine import BaseDataset
+from ..cooking_machine.dataset import BaseDataset
 
 
 # If change, also modify docstring for view()
